#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "engine.h"

/****************************************************************************/
int read_translation_table(char *name, tdatabase *d)
/****************************************************************************/
{
//  int i;
  
  d->translation_table=fopen(name,"rb");

  if (!d->translation_table) {
    printf ("error opening tabfile %s\n",name); 
    exit (1);
  }
  if (fread(&d->tab, sizeof(char), 256, d->translation_table)!=256) {
    fclose (d->translation_table);
    printf("error reading tabfile\n"); 
    exit (1);
  }

  fclose (d->translation_table);
#ifdef DEBUG
  for (int i=0; i<256; i++) printf("%i|",d->tab[i]);
#endif

  return 1;
}

/****************************************************************************/
int read_index_table(char *name, tindex *index)
/****************************************************************************/
{
  index->index_file=fopen(name,"rb");

  if (!index->index_file) {
    printf ("error opening index file\n"); 
    index->is_valid=0;
  }

  if (fread(&index->index_table, sizeof(int), 256, index->index_file)!=256) {
    fclose (index->index_file);
    printf("error reading index_file\n"); 
    index->is_valid=0;
  }
  fclose (index->index_file);

  index->is_valid=1; 

#ifdef DEBUG
  for (i=0; i<256; i++) printf("%i|",index->index_table[i]);
#endif
 
  return 1;
}

/****************************************************************************/
int open_database(char *name, tdatabase *d)
/****************************************************************************/
{
  d->dbffile = fopen(name,"r");
  if (!d->dbffile) {
    printf("couldn't open %s\n",name);
    return 0;
  }
  fread(&d->dbfheader, sizeof(d->dbfheader), 1, d->dbffile);
  return 1;
}

/****************************************************************************/
int use_database(tdatabase *d)
/****************************************************************************/
{
  if (!fseek (d->dbffile, 32, SEEK_SET)) {
    check_fields(d);
    return 1;
  } else return 0;
}

/****************************************************************************/
int check_fields(tdatabase *d)
/****************************************************************************/
{
  int i=0;
  
  do {
#ifdef DEBUG
    printf("reading : %i %i\n", ftell(d->dbffile), i);  
#endif
    fread(&d->field[i], sizeof(d->field[0]), 1, d->dbffile);
  } while (d->field[i++].name[0]!=13);

  d->max_field=i-1;
  return 1;
}

/****************************************************************************/
char * get_column(int which)
/****************************************************************************/
{
  char * col;
  col=(char*) malloc(COLUMN_LENGTH);
  strcpy(col, column[which]);
  return col;
}

/****************************************************************************/
//seek_to_row 
int seek_to_row(int which, tdatabase *d)
/****************************************************************************/
{
  unsigned char buff[100];
  int record_starts=which*d->dbfheader.RecordSize+d->dbfheader.DataOffset+1;
  int pom,i;
  
  fseek (d->dbffile, d->dbfheader.DataOffset+1, SEEK_SET);
#ifdef DEBUG
  printf("pos: %i\n", ftell(d->dbffile));
#endif

  if (which > d->dbfheader.RecordNumber) return 0;
  fseek (d->dbffile, record_starts, SEEK_SET);
  
  for (i=0; i<d->max_field; i++){
    fread(buff, d->field[i].Width, 1, d->dbffile);
    buff[(int)d->field[i].Width]=0;
    if (i==1) {
      for (pom=0; pom<strlen(buff); pom++) {
        buff[pom]=d->tab[buff[pom]];
      }    
    }
    strcpy(column[i], buff);
#ifdef DEBUG
      printf("(%i)%s:'%s', ",strlen(buff), d->field[i].name, buff);
#endif
    }
#ifdef DEBUG
  printf("\n");
#endif
  return 1;
}

/****************************************************************************/
int seek_for(char *string, tdatabase *d)
/****************************************************************************/
{
  int i;

  for (i=0; i<d->dbfheader.RecordNumber; i++) {
    seek_to_row(i, d);
#ifdef DEBUG
    printf("%s, %s\n",column[0],column[1]);
#endif
    if (!strncmp(column[0],string, strlen(string))) printf("found: %s - %s\n", column[0], column[1]);
  }
  return 1;
}

/****************************************************************************/
int get_start(char c, tindex *index)
/****************************************************************************/
{
  return index->index_table[(int)c];
}

/****************************************************************************/
int seek_for_indexed(char *string, tdatabase *d, tindex *index)
/****************************************************************************/
{
  int i=0;
  int was_st_found=0;
  char * column_0;
  char * column_1;
  
  i=index->index_table[(int)string[0]];

  for (; i<d->dbfheader.RecordNumber; i++) {
    seek_to_row(i, d);
    column_0=get_column(0);
    column_1=get_column(1);
#ifdef DEBUG
    printf("%s, %s\n",column_0,column_1);
#endif
    if (!strncmp(column_0,string, strlen(string))) {
      printf("found: %s - %s\n", column_0, column_1);
      was_st_found=1;
    }  
  }
  return was_st_found;
}


/****************************************************************************/
int close_database(tdatabase *d)
/****************************************************************************/
{
  fclose (d->dbffile);
  return 1;
}


/****************************************************************************/
int create_text_index(tdatabase *d)
/****************************************************************************/
{
  int i=0;
  FILE * index;
  char * word=malloc(30);
  
  index=fopen("/tmp/index","w");

  seek_to_row(i, d);
  strcpy(word,column[0]);
  fprintf(index,"%c %i",word[0],0);


  for (i=1; i<d->dbfheader.RecordNumber; i++) {
    seek_to_row(i, d);
#ifdef DEBUG
    printf("%s\n",column[0]);
#endif
    if (column[0][0]>word[0]) {
      strcpy(word,column[0]);
      fprintf(index,"%c %i\n",word[0],i);
      printf("%c %i\n",word[0],i);
    }
  }
  fclose(index);
  return 1;
}

/****************************************************************************/
int create_binary_index(char * index_name)
/****************************************************************************/
{
  unsigned int tab[256];
  char *s=malloc(50);
  int value;

  FILE *f,*fo;
  int i;
  
  f=fopen("/tmp/index","r");
  if (!f) {
    printf ("neni index\n");
    return 0;
  }

  for (i=0; i<256; i++) tab[i]=i;

  while (!feof(f)) {
    fgets(s,50,f);
    value=atoi(s+2);
    printf("%i\n",value);
    tab[(int)s[0]]=value;
  }
  
  fo=fopen(index_name,"wb");
  if (!fo) return 0;
  fwrite(&tab, sizeof(int), 256, fo);
  fclose (fo);  

  return 1;
}

/****************************************************************************/
int create_index(char * database_name, char * index_name)
/****************************************************************************/
{
  tdatabase d;

  open_database(database_name, &d);
  if (use_database(&d)) {
    printf ("use database ok\n");
  } else { 
    printf ("use database failed\n");
    return 0;
  }
  
  create_text_index(&d);
  if (!create_binary_index(index_name)) return 0;
  return 1;
}
